-- Version 0.9.41

DROP TABLE IF EXISTS qbook_metadata;
CREATE TABLE qbook_metadata (
version INTEGER DEFAULT 0,
version_name TEXT,
created_at REAL
);

DROP TABLE IF EXISTS assets;
CREATE TABLE assets (
unique_id INTEGER PRIMARY KEY NOT NULL,
id INTEGER NOT NULL,
identifier TEXT,
type TEXT,
width INTEGER, 
height INTEGER,
duration REAL,
scale REAL,
filename TEXT,
additional_info TEXT,
content BLOB,
byte_size INTEGER,
rendition_info TEXT,
external_resource INTEGER DEFAULT 0,
uid TEXT
);

DROP TABLE IF EXISTS books;
CREATE TABLE books (
id INTEGER PRIMARY KEY NOT NULL,
title TEXT,
description TEXT,
base_width REAL,
base_height REAL,
version INTEGER DEFAULT 0,
version_name TEXT,
-- fixed_layout INTEGER DEFAULT 1,
identifier TEXT,
author_name TEXT,
additional_info TEXT,
uid TEXT
);

DROP TABLE IF EXISTS pages;
CREATE TABLE pages (
id INTEGER PRIMARY KEY NOT NULL,
identifier TEXT,
book_id INTEGER NOT NULL REFERENCES books(id),
width REAL,
height REAL,
page_number INTEGER,
color_background TEXT,
additional_info TEXT,
uid TEXT,
UNIQUE(book_id, page_number)
);

DROP TABLE IF EXISTS elements;
CREATE TABLE elements (
id INTEGER PRIMARY KEY NOT NULL,
identifier TEXT,
type INTEGER,
page_id INTEGER REFERENCES pages(id),
z_order INTEGER,
x REAL,
y REAL,
width REAL,
height REAL,
rotation_angle REAL,
loading_priority INTEGER,
scale REAL,
hidden INTEGER,
background INTEGER DEFAULT 0,
additional_info TEXT,
has_effect INTEGER DEFAULT 0, -- Indicates this element has an associated row of effects
parent_element_id INTEGER REFERENCES elements(id),
-- level_name TEXT, -- e.g. chapter, section, footnote, etc. Mostly internal use (similar to epub:type)
-- reading_order INTEGER, -- sort descriptor for flow layout
-- book_id INTEGER REFERENCES books(id),
uid TEXT
);

DROP TABLE IF EXISTS elements_assets;
CREATE TABLE elements_assets (
element_id INTEGER NOT NULL REFERENCES elements(id),
asset_id INTEGER NOT NULL REFERENCES assets(id),
sort_order INTEGER,
additional_info TEXT,
PRIMARY KEY (element_id, asset_id, sort_order)
);

DROP TABLE IF EXISTS filters;
CREATE TABLE filters (
id INTEGER PRIMARY KEY NOT NULL,
name TEXT,
default_display_name TEXT,
description TEXT,
language_filter INTEGER NOT NULL DEFAULT 0,
sort_order INTEGER NOT NULL DEFAULT 0,
is_locked INTEGER DEFAULT 0,
unlock_information TEXT, -- can hold In App Product ID (SKU)
UNIQUE(name) ON CONFLICT IGNORE
);

DROP TABLE IF EXISTS elements_filters;
CREATE TABLE elements_filters (
element_id INTEGER  NOT NULL REFERENCES elements(id),
filter_id INTEGER  NOT NULL REFERENCES filters(id),
PRIMARY KEY (element_id, filter_id)
);

DROP TABLE IF EXISTS text_attributes;
CREATE TABLE text_attributes (
id INTEGER PRIMARY KEY NOT NULL,
asset_id INTEGER NOT NULL REFERENCES assets(id),
location INTEGER,
length INTEGER,
font_asset_id INTEGER REFERENCES assets(id), 
font_name TEXT,
font_size REAL,
kern REAL,
color_foreground TEXT,
ligature INTEGER DEFAULT 0,
-- overrides_default DEFAULT 1,
stroke INTEGER DEFAULT 0,
stroke_width REAL DEFAULT 0,
stroke_color TEXT,
shadow INTEGER DEFAULT 0,
shadow_color TEXT,
shadow_offset_x REAL DEFAULT 0,
shadow_offset_y REAL DEFAULT 0,
shadow_blur_radius REAL DEFAULT 0,
status INTEGER
);

DROP TABLE IF EXISTS audio_options;
CREATE TABLE audio_options (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER  NOT NULL REFERENCES elements(id),
loop INTEGER,
auto_start INTEGER DEFAULT 1,
positioning INTEGER DEFAULT 0,
fade_out_range REAL DEFAULT 0,
full_volume_radius REAL DEFAULT 0,
UNIQUE(element_id) ON CONFLICT REPLACE
);

DROP TABLE IF EXISTS animation_options;
CREATE TABLE animation_options (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER  NOT NULL REFERENCES elements(id),
frame_interval INTEGER,
loop INTEGER,
loop_interval INTEGER,
freezes_when_done INTEGER,
auto_start INTEGER DEFAULT 1,
reversible INTEGER DEFAULT 0,
UNIQUE(element_id) ON CONFLICT REPLACE
);

DROP TABLE IF EXISTS video_options;
CREATE TABLE video_options (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER  NOT NULL REFERENCES elements(id),
fullscreen_allowed INTEGER DEFAULT 1,
loop INTEGER DEFAULT 0,
autostart INTEGER DEFAULT 0,
fullscreen_start INTEGER DEFAULT 0,
embedded INTEGER DEFAULT 1,
start REAL DEFAULT 0,
duration REAL DEFAULT -1,
usage_type INTEGER DEFAULT 0,
usage_info TEXT,
UNIQUE(element_id) ON CONFLICT REPLACE
);


DROP TABLE IF EXISTS basic_interactions;
CREATE TABLE basic_interactions (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER  NOT NULL REFERENCES elements(id),
playable INTEGER,
stoppable INTEGER,
draggable INTEGER,
priority INTEGER,
command TEXT,
exclusive INTEGER DEFAULT 0,
disable_others INTEGER DEFAULT 0
);

DROP TABLE IF EXISTS text_timings;
CREATE TABLE text_timings (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER  NOT NULL REFERENCES elements(id),
name TEXT,
start REAL,
duration REAL,
location INTEGER DEFAULT -1,
length INTEGER DEFAULT 0,
track_number INTEGER DEFAULT 1,
command TEXT
);

DROP TABLE IF EXISTS sub_texts;
CREATE TABLE sub_texts (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER  NOT NULL REFERENCES elements(id),
parent_sub_text_id INTEGER REFERENCES sub_texts(id),
asset_id INTEGER REFERENCES assets(id),
name TEXT,
location INTEGER,
length INTEGER,
track_number INTEGER DEFAULT 1,
origin_x REAL,
origin_y REAL,
x REAL,
y REAL,
width REAL,
height REAL,
prerendered INTEGER DEFAULT 0,
presentation_asset_id INTEGER REFERENCES assets(id),
command TEXT
);

DROP TABLE IF EXISTS page_thumbs;
CREATE TABLE page_thumbs (
id INTEGER PRIMARY KEY NOT NULL,
page_id INTEGER NOT NULL REFERENCES pages(id),
asset_id INTEGER NOT NULL REFERENCES assets(id)
);

DROP TABLE IF EXISTS page_thumbs_filters;
CREATE TABLE page_thumbs_filters (
page_thumb_id INTEGER  NOT NULL REFERENCES page_thumbs(id),
filter_id INTEGER  NOT NULL REFERENCES filters(id),
PRIMARY KEY(page_thumb_id, filter_id)
);

DROP TABLE IF EXISTS extra_modules;
CREATE TABLE extra_modules (
id INTEGER PRIMARY KEY NOT NULL,
identifier TEXT,
type_description TEXT,
extra_options TEXT,
book_id INTEGER  NOT NULL REFERENCES books(id)
-- UNIQUE(book_id, type_description)
);

DROP TABLE IF EXISTS extra_modules_filters;
CREATE TABLE extra_modules_filters (
extra_module_id INTEGER  NOT NULL REFERENCES extra_modules(id),
filter_id INTEGER  NOT NULL REFERENCES filters(id),
PRIMARY KEY(extra_module_id, filter_id)
);

DROP TABLE IF EXISTS extra_modules_assets;
CREATE TABLE extra_modules_assets (
extra_module_id INTEGER NOT NULL REFERENCES extra_modules(id),
asset_id INTEGER NOT NULL REFERENCES assets(id),
sort_order INTEGER,
type_description TEXT,
additional_info TEXT,
PRIMARY KEY (extra_module_id, asset_id, sort_order)
);

DROP TABLE IF EXISTS effects;
CREATE TABLE effects (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER NOT NULL REFERENCES elements(id),
identifier TEXT,
type_description TEXT, -- accel: Use Accelerometer, gyro: Use gyro, accel;gyro: Use Both, etc.
floating_radius REAL DEFAULT 0,
trace_touch INTEGER DEFAULT 0,
limit_radius REAL DEFAULT 0,
reverse_x INTEGER DEFAULT 0, -- Only for Accelerometer
reverse_y INTEGER DEFAULT 0,
range_mult_x REAL DEFAULT 1,
range_mult_y REAL DEFAULT 1,
stack_z_pos REAL, -- For migration purpose only. Ignore this for now.
stack_master INTEGER DEFAULT 0, -- For migration purpose only. Ignore this for now.
extra_options TEXT
);

-- DROP TABLE IF EXISTS navigation_infos;
-- CREATE TABLE navigation_infos (
-- id INTEGER PRIMARY KEY NOT NULL,
-- element_id INTEGER NOT NULL REFERENCES elements(id),
-- depth INTEGER, -- for indentation hint (pre-calculated by the tool)
-- title TEXT,
-- extra_options TEXT -- reserved for future use
-- );

-- DROP TABLE IF EXISTS flow_layout_options;
-- CREATE TABLE flow_layout_options (
-- id INTEGER PRIMARY KEY NOT NULL,
-- name TEXT,
-- fill_options INTEGER DEFAULT 0,
-- content_gravity INTEGER DEFAULT 0,
-- padding_left REAL DEFAULT 0.0,
-- padding_right REAL DEFAULT 0.0,
-- padding_top REAL DEFAULT 0.0,
-- padding_bottom REAL DEFAULT 0.0,
-- text_wrap_option INTEGER DEFAULT 0,
-- resizable_options INTEGER DEFAULT 0,
-- extra_options TEXT
-- );

-- DROP TABLE IF EXISTS elements_flow_layout_options;
-- CREATE TABLE elements_flow_layout_options (
-- element_id INTEGER NOT NULL REFERENCES elements(id),
-- flow_layout_options_id INTEGER NOT NULL REFERENCES flow_layout_options(id)
-- );

DROP TABLE IF EXISTS paragraph_attributes;
CREATE TABLE paragraph_attributes (
id INTEGER PRIMARY KEY NOT NULL,
asset_id INTEGER NOT NULL REFERENCES assets(id),
location INTEGER,
length INTEGER,
alignment INTEGER, -- 0: left, 1: right, 2: centre, 3: adjust
indent_first REAL, -- indentation of the first line
indent_head REAL,
indent_tail REAL,
line_spacing REAL,
paragraph_spacing REAL,
paragraph_spacing_before REAL, -- Reserved
max_line_height REAL, -- Reserved
min_line_height REAL, -- Reserved
line_height_multiple REAL,  -- Reserved
line_break_mode INTEGER,  -- Reserved
hyphenation_factor REAL DEFAULT 0.0, -- Reserved
tightening_factor_for_truncation REAL, -- Reserved
default_tab_interval REAL, -- Reserved
tab_stops TEXT -- Reserved
);

DROP TABLE IF EXISTS book_options;
CREATE TABLE book_options (
book_id INTEGER  NOT NULL REFERENCES elements(books),
key TEXT,
is_on INTEGER,
value TEXT
);

DROP TABLE IF EXISTS filter_options;
CREATE TABLE filter_options (
book_id INTEGER NOT NULL REFERENCES elements(books),
filter_id INTEGER NOT NULL REFERENCES elements(filters),
key TEXT,
is_on INTEGER,
value TEXT
);

DROP TABLE IF EXISTS element_group;
CREATE TABLE element_group (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER NOT NULL REFERENCES elements(id),
group_order INTEGER NOT NULL,
page_id INTEGER NOT NULL REFERENCES pages(id)
);

DROP TABLE IF EXISTS comics_effects;
CREATE TABLE comics_effects (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER NOT NULL REFERENCES elements(id),
alpha REAL,
alphaEnd REAL,
alphaDuration INTEGER,
ambient INTEGER,
anchorx INTEGER,
anchory INTEGER,
x1 INTEGER,
y1 INTEGER,
x2 INTEGER,
y2 INTEGER,
translateReverse INTEGER,
translateDuration INTEGER,
translateEnabled INTEGER,
controlPointEnabled INTEGER,
parallaxEnabled INTEGER,
parallaxIntensity REAL,
zvalue INTEGER,
extra_options TEXT
);

CREATE TABLE pages_filters (
page_id INTEGER  NOT NULL REFERENCES pages(id),
filter_id INTEGER  NOT NULL REFERENCES filters(id),
PRIMARY KEY (page_id, filter_id)
);

DROP TABLE IF EXISTS viewports;
CREATE TABLE viewports (
id INTEGER PRIMARY KEY NOT NULL,
page_id INTEGER  NOT NULL REFERENCES pages(id),
identifier TEXT,
x REAL,
y REAL,
width REAL,
height REAL,
rotation_angle REAL, -- in radians
zoom_in_allowed INTEGER DEFAULT 0,
zoom_out_allowed INTEGER DEFAULT 0,
preferred_fit INTEGER DEFAULT 0 -- ignored if zoom_in_allowed is false
);

CREATE TABLE event_trigger_options (
id INTEGER PRIMARY KEY NOT NULL,
element_id INTEGER  NOT NULL REFERENCES elements(id),
autostart INTEGER DEFAULT 0,
repeat INTEGER DEFAULT 1,
target_identifiers TEXT,
options TEXT,
UNIQUE(element_id) ON CONFLICT REPLACE
);

CREATE INDEX assets_id_idx ON assets (id);
CREATE INDEX text_timings_start_idx ON text_timings (start);
CREATE INDEX text_timings_location_idx ON text_timings (location);
CREATE INDEX sub_texts_location_idx ON sub_texts (location);
CREATE INDEX text_attributes_location_idx ON text_attributes (location);
CREATE INDEX paragraph_attributes_location_idx ON paragraph_attributes (location);

